import chipwhisperer as cw
import time
from tqdm import tqdm
import numpy as np
import matplotlib.pyplot as pl


print(cw.__file__)
scope = cw.scope()
scope.default_setup()



target = cw.target(scope)

SCOPETYPE = 'OPENADC'
PLATFORM = 'CWLITEARM'
CRYPTO_TARGET = 'TINYAES128C'
num_traces = 20000
num_iter = 50
CHECK_CORR = False



if "STM" in PLATFORM or PLATFORM == "CWLITEARM" or PLATFORM == "CWNANO":
    prog = cw.programmers.STM32FProgrammer
elif PLATFORM == "CW303" or PLATFORM == "CWLITEXMEGA":
    prog = cw.programmers.XMEGAProgrammer
else:
    prog = None


time.sleep(0.05)
scope.default_setup()
def reset_target(scope):
    if PLATFORM == "CW303" or PLATFORM == "CWLITEXMEGA":
        scope.io.pdic = 'low'
        time.sleep(0.05)
        scope.io.pdic = 'high_z' #XMEGA doesn't like pdic driven high
        time.sleep(0.05)
    else:  
        scope.io.nrst = 'low'
        time.sleep(0.05)
        scope.io.nrst = 'high'
        time.sleep(0.05)
        
fw_path = 'simpleserial-aes-{}.hex'.format(PLATFORM)
cw.program_target(scope, prog, fw_path)


ktp = cw.ktp.Basic()
ktp._fixedKey = True
ktp._fixedPlain = True



for j in range(num_iter):

    traces = []
    keys = []
    texts = []
    scope.gain.db = 25

    for i in tqdm(range(num_traces), desc='Capturing traces'):
	
        
        key, text = ktp.next()  # manual creation of a key, text pair can be substituted here
	
        trace = cw.capture_trace(scope, target, text, key)
        if trace is None:
            continue
        traces.append(trace)

    #Convert traces to numpy arrays
    trace_array = np.asarray([trace.wave for trace in traces])  # if you prefer to work with numpy array for number crunching
    textin_array = np.asarray([trace.textin for trace in traces])
    known_keys = np.asarray([trace.key for trace in traces])  # for fixed key, these keys are all the same
    
	trace_name = "traces.npy"

    np.save(trace_name, trace_array)

    del trace_array
    del textin_array
    del known_keys




